#!/bin/bash
# treebeard.cgi 
# created and copyrighted: 2 Jan 11
# author: Evsiz (evsiz@this-voice.org)
# The author releases this file under the GNU GPL
# license. See GNU's COPYING file for specifics.
#

# consts
ROOT=fanghorn
INDEX=_everybody
CGI=treebeard.cgi
RTPATH=`pwd`/$ROOT
TFILE=$RTPATH/heart/topics # not in a dir with a _rootpage
UTF8="" # TRUE to use UTF-8 (unsupported)
BANNED_IPS="x"  # space between IPs - keep x at end of list

decode () {
    arg=`echo $1 | awk '{ 
       gsub(/\%0D\%0A/, "\r") ;
       gsub(/\%0d\%0a/, "\r") ;
       gsub(/\x2B/, " ") ; 
       gsub(/\x2b/, " ") ; 
       print
    }'`
    i="0"
    while [ "$i" -lt ${#arg} ]; do
        c0=${arg:$i:1}
        if [ "x$c0" = "x%" ]; then
            c1=${arg:$((i+1)):1}
            c2=${arg:$((i+2)):1}
            printf "\x$c1$c2"
            i=$((i+3))
        else
            echo -n "$c0"
            i=$((i+1))
        fi
    done

}

err_pg () {
    echo "<hr><center>LOST IN FANGHORN FOREST.<p>"$1"</center>"
}


parse_val () {
    echo $1 | awk 'BEGIN { RS = "&" } ; \
	{ if (index($0, "'$2'") == 1) { split($0, tmp, "="); print tmp[2]; exit; }};'
}

pwd2dir () {
    if [ "$1" == "" ]; then
	echo "none"
	return
    fi
    cat $TFILE | awk 'BEGIN { FS = ":" } ; \
        { if ($1 ~ "'$1'") { print $2 ; exit }}'
}

## main ##
echo "Content-Type: text/html

"

if [ "$UTF8" == "TRUE" ]; then
	echo "<!DOCTYPE html>
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>"
fi

REMOTE=$HTTP_X_CLUSTER_CLIENT_IP 
if [ "$REMOTE" == "" ]; then REMOTE=$REMOTE_ADDR ; fi

BANNED=`echo $BANNED_IPS | awk 'BEGIN { RS = " " }; { \
    if ( index( "'$REMOTE'", $0 ) == 1 ) { print "banned"; exit } }'`
if [ "$BANNED" == "banned" ]; then
    err_pg "No Orcs Allowed<hr>"
    exit
fi

REQUEST=`echo  $REQUEST_URI | cut -d"?" -f2 | tr -d "*"`
TYPE=`echo  $REQUEST | cut -d"=" -f1` # must be first form entry

# PAGE
if [ "$TYPE" == "page" ]; then
    PAGE=`echo $(parse_val $REQUEST "page") | sed 's/%2F/\//g' | tr -d .\*`
    TPSWD=$(parse_val $REQUEST "user")
    # handle incoming writes from "alter" or "grow"
    if [ "$TPSWD" != "" ]; then
	MONTH=$(parse_val $REQUEST "month")
	DIR=`dirname $PAGE`
	TDIR=$(pwd2dir $TPSWD)
	TEXT=$(parse_val $REQUEST "content")
	if [ -e $RTPATH/$PAGE ] && [ ! -w $RTPATH/$PAGE ]; then
	    err_pg "Page is read-only.<br>Write Cancelled."
	elif [ "$MONTH" != `date +%m` ]; then
	    err_pg "Are you calendarically impaired?<br>Write Cancelled."
	elif [ "$TDIR" != "$DIR" ]; then
	    err_pg "Wrong password for this topic.<br>Write Cancelled."
	else
	    HTML=`echo $TEXT | awk 'BEGIN { RS = "\a" } ; \
		{ if ($0 ~ /\%3C/) print "FOUND"  } '`
	    LEN=`echo $TEXT | awk 'BEGIN { RS = "\a" } ; \
		{ if (length($0) < 5000) print "OK" } '`
	    if [ "$HTML" == "FOUND" ]; then
			err_pg "HTML detected - No tags permitted."
	    elif [ "$LEN" != "OK" ]; then
			err_pg "Text submission is over 5000 bytes."
	    else
			CONTENT=$(decode $TEXT)	
			echo $CONTENT | awk 'BEGIN { RS = "\r" } \
		    {   gsub("\r", "\n") ; 
			print $0
			}' > $RTPATH/$PAGE  # write to file
			TS=`date +%x" "%X`
			echo $REMOTE" "$TS >> $RTPATH/$PAGE
			PG=`basename $PAGE`
			echo " <p>
<a href=/cgi-bin/treebeard.cgi?page="$PAGE">"$PG"</a> - 
<a href=/cgi-bin/treeadmin.cgi?warn="$PAGE">warn</a> -
<a href=/cgi-bin/treeadmin.cgi?delete="$PAGE">delete</a><br>" >> `pwd`/write.log
			fi
		fi
    elif [ ! -f $RTPATH/$PAGE ]; then
        PAGE=$INDEX
    fi

    TITLE=`echo $PAGE | tr -d _ | awk -F/ '{ print $NF }' | sed 's/-/ /g'`
    echo "<title>"$TITLE"</title> 
<h4>"$TITLE"</h4><hr><pre>"
    cat $RTPATH/$PAGE
    echo "</pre>"
    DIR=`dirname $PAGE`
    if [ "$DIR" != "." ] && [ "$DIR" != "/" ]; then
	echo "
<hr><form method=\"get\" action=\"/cgi-bin/"$CGI"\">
<input type=\"hidden\" name=\"alter\" value=\""$PAGE"\" >
If this is your page, you can edit it by entering the topic password.<br>
Edit Password: 
<input type=\"password\" name=\"user\" size=\"16\" maxlength=\"32\">
<input type=\"submit\" value=\"Submit\"></form>"
	echo "<hr><form method=\"get\" action=\"/cgi-bin/"$CGI"\">
You can grow a new page for a topic by entering its topic password.<br>
New Page Name:
<input type=\"text\" name=\"grow\" size=\"32\" maxlength=\"32\">
Grow Password: 
<input type=\"password\" name=\"user\" size=\"16\" maxlength=\"32\">
<input type=\"submit\" value=\"Submit\"></form>"
    fi

# JOIN
elif [ "$TYPE" == "join" ]; then
    MONTH=$(parse_val $REQUEST "month")
    TOPIC=`echo $(parse_val $REQUEST "join") | tr -d _+% \
| tr [:upper:] [:lower:]`
    EMAIL=$(parse_val $REQUEST "email")
    if [ "$MONTH" != `date +%m` ]; then
	err_pg "Are you calendarically impaired?"
    elif [ "$TOPIC" == "" ]; then
	err_pg "What's the deal with your topic name?"
    elif [ "$EMAIL" == "" ]; then
	err_pg "What's the deal with your email?"
    else
	echo "<title>Registering New Topic</title> 
<h4>Registering New Topic</h4><hr>"
	TDIR=`date +%y`"/"`date +%j`"/"`date +%H%M%S`
	PASSWD=`date +%s`
	PASSWD=`echo $PASSWD | tr 1234 IuEa`
	TLINE=$PASSWD":"$TDIR":"$EMAIL
	echo $TLINE >> $TFILE
	mkdir -p $RTPATH/$TDIR
	TS=`date +%x" "%X`
	echo "DELETE
New Topic: "$TOPIC"
Remove this DELETE tag and timestamp and add your content.
<p>"$REMOTE" "$TS"" > $RTPATH/$TDIR/"_"$TOPIC
	echo "<p>Welcome to our topical forest!
<p>You should see your new topic, with an underscore prepended,
in the links of branches below.  If there is more than one of these
topics, yours is the one with a New Topic marker in it.

<p>Below is your topic password for that directory.  This password
gives anyone access to grow and alter the pages there.  Go put the
password in the branch page's Edit link and start growing.  Or put
this topic password in any grow link and add a page to your topic.

<p>Your "$TOPIC" password: "$PASSWD"

<p>You can keep this topic's password secret or you can share it with
your woodland friends.  BUT WRITE IT DOWN AND DON'T LOSE IT!"
    fi

# ALTER
elif [ "$TYPE" == "alter" ]; then
    PAGE=`echo $(parse_val $REQUEST "alter") | sed 's/%2F/\//g'`
    TPSWD=$(parse_val $REQUEST "user")
    DIR=`dirname $PAGE`
    TDIR=$(pwd2dir $TPSWD)
    if [ "$TDIR" != "$DIR" ]; then
	err_pg "Wrong topic password for this page."
    else
	DFLT=`cat $RTPATH/$PAGE`
	NAME=`basename $PAGE`
	echo "<title>Altering Existing Page</title> 
<h4>Altering Page: "$NAME"</h4><hr>
<form method=\"get\" action=\"/cgi-bin/"$CGI"\">
<input type=\"hidden\" name=\"page\" value=\""$PAGE"\" >
<input type=\"hidden\" name=\"user\" value=\""$TPSWD"\" >
Page published as-is to preserve white space.<br>
Page size limited to size of form WITHOUT SCROLLBARS.<br>
No HTML tags allowed - HTML entities are okay.
<p align=\"left\"><textarea name=\"content\" rows=\"70\" cols=\"70\">
$DFLT</textarea>
<p>Just to make sure you are actually here, if January is 01
and February is 02, what month is it now? <input type=\"text\"
name=\"month\" size=\"2\" maxlength=\"2\">
<p><input type=\"submit\" value=\"Submit\"></form>"
    fi

# GROW
elif [ "$TYPE" == "grow" ]; then
    PNAME=`echo $(parse_val $REQUEST "grow") | tr -d _+% \
| tr [:upper:] [:lower:]`
    TPSWD=$(parse_val $REQUEST "user")
    TDIR=$(pwd2dir $TPSWD)
    if [ "$TDIR" == "" ]; then
	err_pg "Password not in database."
    elif [ "$PNAME" == "" ]; then
	err_pg "What's the deal with your new page name?"
    else
    echo "<title>Creating New Page</title> 
<h4>Creating Page: "$PNAME"</h4><hr>
<form method=\"get\" action=\"/cgi-bin/"$CGI"\">
<input type=\"hidden\" name=\"page\" value=\""$TDIR"/"$PNAME"\" >
<input type=\"hidden\" name=\"user\" value=\""$TPSWD"\" >
Page published as-is to preserve white space.<br>
Page size limited to size of form WITHOUT SCROLLBARS.<br>
No HTML tags allowed - HTML entities are okay.
<p align=\"left\"><textarea name=\"content\" rows=\"70\" cols=\"70\">
</textarea>
<p>Just to make sure you are actually here, if January is 01
and February is 02, what month is it now? <input type=\"text\"
name=\"month\" size=\"2\" maxlength=\"2\">
<p><input type=\"submit\" value=\"Submit\"></form>"
    fi

# Bad Requests
else 
    err_pg "And the trees are angry ..."
fi

# Menu Current Dir
echo "<hr>
<table width=90%><tr valign=top>
<td><p>Leaves<br>"
PWD=`dirname $PAGE 2>/dev/null | tr -d .`
LEAVES=`ls -1 $ROOT/$PWD`
for leaf in $LEAVES; do
    if [[ -f $RTPATH/$PWD/$leaf ]] ; then
	echo "<a href=/cgi-bin/"$CGI"?page=$PWD/$leaf>$leaf</a><br>"
    fi
done

# Menu All Dirs
echo "<td><p>Branches<br>"
echo `ls -1R $ROOT` | awk 'BEGIN { RS = " " } ; \
    {if ($0 ~ /:/) dir = $0 ; else if ($0 ~ /_/) \
    {print "<a href=/cgi-bin/'$CGI'?page="dir $0">"$0"</a><br>" \
    | "tr : /" } } ' | sed 's/'$ROOT'\///g'
